package;

import js.Browser;
import js.html.CanvasElement;
import js.html.CanvasRenderingContext2D;
import js.html.Element;
import js.html.Event;
import js.html.HTMLDocument;
import js.html.KeyboardEvent;

import mje.*;
import mje.MJEData.Vec2;
import mje.MJEData.Vector2;

typedef Paddle = {x:Float, y:Float, width:Float, height:Float}
typedef Ball = {x:Float, y:Float, radius:Float, vel:Vector2, reset:Void -> Void}

/**
 * ...
 * @author JanGamesCo
 */
class Main extends MJEGame
{	
	//Paddle
	private var playerPos:Paddle;
	private var oponentPos:Paddle;
	
	//Points
	private var paddleSize:Vec2;
	private var playerPoints:Int;
	private var oponentPoints:Int;
	
	//Ball
	private var ball:Ball;
	
	static function main() 
	{
		new Main();
	}
	
	//Contructor
	public function new()
	{
		super();
	}
	
	//Init
	override public function init() 
	{
		//Paddle size
		paddleSize = { x: 32, y: 128 };
		
		//Init player and enemy
		playerPos = { x: paddleSize.x, y: canvas.height / 2, width: paddleSize.x, height: paddleSize.y };
		playerPos.y -= playerPos.height / 2;
		oponentPos = { x: canvas.width - paddleSize.x * 2, y: canvas.height / 2, width: paddleSize.x, height: paddleSize.y };
		oponentPos.y -= oponentPos.height / 2;
		
		//Init ball
		ball = { x: canvas.width / 2, y: canvas.height / 2, radius: 16, vel: new Vector2(0.1, 0), reset: ballReset };
		ball.x -= ball.radius / 2;
		ball.y -= ball.radius / 2;
		ball.vel.x *= (Math.random() < 0.5 ? -1 : 1);
		ball.vel.y = (Math.random() < 0.5 ? -1 : 1) * (Math.random() * 0.4 + 0.1);
		
		//Init points
		playerPoints = 0;
		oponentPoints = 0;
		
		super.init();
	}
	
	override public function draw() 
	{
		super.draw();
		
		//White
		ctx.fillStyle = "#FFFFFF";
		
		//Draw player paddle
		ctx.fillRect(playerPos.x, playerPos.y, playerPos.width, playerPos.height);
		
		//Draw enemy paddle
		trace(oponentPos);
		ctx.fillRect(oponentPos.x, oponentPos.y, oponentPos.width, oponentPos.height);
		
		//Draw ball
		ctx.beginPath();
		ctx.arc(ball.x + ball.radius, ball.y + ball.radius, ball.radius, 0, Math.PI * 2, false);
		ctx.fill();
		ctx.closePath();
		
		//Draw text
		ctx.save();
		ctx.translate(canvas.width / 3, canvas.height / 3);
		ctx.scale(10, 10);
		ctx.fillText(playerPoints + "", 0, 0);
		ctx.restore();
		ctx.save();
		ctx.translate(canvas.width - canvas.width / 3, canvas.height / 3);
		ctx.scale(10, 10);
		ctx.fillText(oponentPoints + "", 0, 0);
		ctx.restore();
	}
	
	override function update() 
	{
		super.update();
		
		//Update
		ball.x += ball.vel.x * delta * screenScale.x;
		ball.y += ball.vel.y * delta * screenScale.y;
		//Ball Y collision
		if (ball.y < 0) {
			ball.vel.y *= -1;
			ball.y = 0;
		} else if(ball.y + ball.radius * 2 > canvas.height) {
			ball.vel.y *= -1;
			ball.y = canvas.height - ball.radius * 2;
		}
		//Ball X collision
		if (ball.x < 0) {
			oponentPoints++;
			ball.reset();
		} else if(ball.x + ball.radius > canvas.width) {
			playerPoints++;
			ball.reset();
		}
		//Ball paddle collision
		if (ballPaddle(playerPos)) {
			ball.vel.x *= -1.01;
			ball.vel.y *= 1.01;
			ball.x = playerPos.x + playerPos.width;
		} else if (ballPaddle(oponentPos)) {
			ball.vel.x *= -1;
			ball.vel *= 1.01;
			ball.x = oponentPos.x - ball.radius * 2;
		}
		
		//Player movement
		if (MJE.the.keyDown("S")) {
			playerPos.y += delta * 0.4 * screenScale.y;
		} else if (MJE.the.keyDown("W")) {
			playerPos.y -= delta * 0.4 * screenScale.y;
		}
		if (playerPos.y < 0) { playerPos.y = 0; }
		else if (playerPos.y + playerPos.height > canvas.height) { playerPos.y = canvas.height - playerPos.height; }
		
		//Oponent movement
		var dist:Float = Math.abs(oponentPos.y - ball.y);
		if (ball.y > oponentPos.y + oponentPos.height / 2) {
			oponentPos.y += delta * 0.4 * screenScale.y * Math.min(dist / oponentPos.height, 1);
		} else {
			oponentPos.y -= delta * 0.4 * screenScale.y * Math.min(dist / oponentPos.height, 1);
		}
		if (oponentPos.y < 0) { oponentPos.y = 0; }
		else if (oponentPos.y + oponentPos.height > canvas.height) { oponentPos.y = canvas.height - oponentPos.height; }
	}
	
	private function ballReset()
	{
		ball.x = canvas.width / 2;
		ball.y = canvas.height / 2;
		ball.x -= ball.radius / 2;
		ball.y -= ball.radius / 2;
		
		ball.vel.x = 0.3 * (ball.vel.x < 0 ? -1 : 1);
		ball.vel.y = (Math.random() < 0.5 ? -1 : 1) * (Math.random() * 0.4 + 0.1);
	}
	
	private function ballPaddle(paddle:Paddle):Bool
	{
		if (ball.x < paddle.x + paddle.width &&
			ball.x + ball.radius * 2 > paddle.x &&
			ball.y < paddle.y + paddle.height &&
			ball.y + ball.radius * 2 > paddle.y) {
			return true;
		}
		
		return false;
	}
	
	override private function resize() 
	{
		super.resize();
		
		//Resize positions
		playerPos.x = paddleSize.x * screenScale.x; playerPos.y = playerPos.y * screenDiff.y;
		playerPos.width = paddleSize.x * screenScale.x; playerPos.height = paddleSize.y * screenScale.y;
		
		oponentPos.x = canvas.width - paddleSize.x * 2 * screenScale.x; oponentPos.y = oponentPos.y * screenDiff.y;
		oponentPos.width = paddleSize.x * screenScale.x; oponentPos.height = paddleSize.y * screenScale.y;
		
		ball.x = ball.x * screenDiff.x; ball.y = ball.y * screenDiff.y;
		ball.radius = 16 * screenScale.x;
	}
}