package mje;

import js.Browser;
import js.html.Element;
import js.html.Event;
import js.html.HTMLDocument;
import js.html.KeyboardEvent;
import js.html.CanvasElement;
import js.html.CanvasRenderingContext2D;

import mje.MJEData.Vec2;

/**
 * ...
 * @author JanGamesCo
 */
class MJEGame extends MJEGroup<MJEBasic>
{
	//Key constants
	public static inline var UP:Int = 0;
	public static inline var DOWN:Int = 1;
	public static inline var PRESSED:Int = 2;
	
	//Element references
	public var doc:HTMLDocument;
	public var body:Element;
	/** Canvas reference */
	public var canvas:CanvasElement;
	/** Context reference */
	public var ctx:CanvasRenderingContext2D;
	
	//Time
	private var isPaused:Bool;
	public var delta:Float;
	public var lastTime:Float;
	
	//Screen vars
	private var initScreenSize:Vec2;
	private var screenScale:Vec2;
	private var lastSize:Vec2;
	private var screenDiff:Vec2;
	
	//Keybard array
	public var keyArray:Array<Int>;
	private var keyDownStack:Array<Int>;
	
	//ID
	private var lastID:Int;

	public function new() 
	{		
		//Get document
		doc = Browser.window.document;
		
		//Get body
		body = doc.body;
		body.style.margin = "0";
		body.style.overflow = "hidden";
		
		//Setup MJE
		new MJE();
		
		//Setup MJE game reference
		MJE.the.game = this;
		delta = 0;
		MJE.delta = 0;
		
		//
		lastTime = 0;
		
		//Setup last ID and ID
		lastID = 0;
		
		//Create canvas
		canvas = doc.createCanvasElement();
		canvas.width = Browser.window.innerWidth;
		canvas.height = Browser.window.innerHeight;
		body.appendChild(canvas);
		MJE.the.canvas = canvas;
		
		//Canvas context
		ctx = canvas.getContext2d();
		MJE.the.ctx = ctx;
		
		//Key array
		keyArray = new Array();
		keyDownStack = new Array();
		
		//Init screen size
		//initScreenSize = { x: canvas.width, y: canvas.height };
		initScreenSize = { x: 1920, y: 1080 };
		screenScale = { x: 1, y: 1 };		
		
		//Events listeners
		Browser.window.onresize = resize;
		Browser.window.onload = init;
		Browser.window.onblur = pause;
		Browser.window.onfocus = resume;
		Browser.window.onkeydown = onKeyDown;
		Browser.window.onkeyup = onKeyUp;
		
		super();
	}
	
	//Onload event handler
	public function init()
	{
		resize();
		render(-1);
	}
	
	//Render event handler
	private function render(Time:Float)
	{
		//Drawing
		//Clear screen
		ctx.clearRect(0, 0, canvas.width, canvas.height);
		ctx.fillStyle = "#000000";
		ctx.fillRect(0, 0, canvas.width, canvas.height);
		
		//Draw total time and delta
		ctx.fillStyle = "#FFFFFF";
		ctx.fillText("Total time: " + Math.floor(Time * 10) / 10000 + "s", 10, 10);
		ctx.fillText("Delta time: " + Math.floor(MJE.delta * 1000) / 1000 + "s", 10, 20);
		ctx.fillText("Canvas Size: (x: " + canvas.width + ", y: " + canvas.height + ")", 10, 30);
		ctx.fillText("Scale from initial size: (x: " + Math.floor(screenScale.x * 100) / 100 + 
			", y: " + Math.floor(screenScale.y * 100) / 100 + ")", 10, 40);
		
		//Draw
		draw();
		
		//Paused screen
		if (isPaused) {
			ctx.fillStyle = "#000000";
			ctx.globalAlpha = 0.5;
			ctx.fillRect(0, 0, canvas.width, canvas.height);
			ctx.globalAlpha = 1;
			
			ctx.fillStyle = "#FFFFFF";
			ctx.beginPath();
			ctx.moveTo(canvas.width * 0.4, canvas.height * 0.3);
			ctx.lineTo(canvas.width * 0.6, canvas.height * 0.5);
			ctx.lineTo(canvas.width * 0.4, canvas.height * 0.7);
			ctx.lineTo(canvas.width * 0.4, canvas.height * 0.3);
			ctx.fill();
		}
		
		//Update
		//Get delta
		if(Time > 0) {
			delta = Time - lastTime;
			MJE.delta = delta / 1000;
		}
		
		//Paused
		if (isPaused) { lastTime = Time; }
		else {
			update();
		}
		
		//Update input keydown
		while (keyDownStack.length > 0) {
			var code:Int = keyDownStack.pop();
			keyArray[code] = DOWN;
		}
		
		//Get last time
		lastTime = Time;
		
		//Request loop animation frame
		Browser.window.requestAnimationFrame(render);
	}
	
	//Draw
	override public function draw()
	{		
		super.draw();
	}
	
	//Update
	override public function update()
	{		
		super.update();
	}
	
	//Get ID
	public function getID():Int
	{
		return lastID++;
	}
	
	//On resize event handler
	private function resize()
	{
		lastSize = { x: canvas.width, y: canvas.height };
		
		//Resize canvas
		canvas.width = Browser.window.innerWidth;
		canvas.height = Browser.window.innerHeight;
		
		screenScale = { x: canvas.width / initScreenSize.x, y: canvas.height / initScreenSize.y };
		screenDiff = { x: canvas.width / lastSize.x, y: canvas.height / lastSize.y };
	}
	
	private function onKeyDown(event:KeyboardEvent)
	{
		var code:Int = event.keyCode;
		if(keyArray[code] == 0) {
			keyArray[code] = PRESSED;
		}
		keyDownStack.push(code);
	}
	
	private function onKeyUp(event:KeyboardEvent)
	{
		var code:Int = event.keyCode;
		keyArray[code] = UP;
	}
	
	private function pause()
	{
		isPaused = true;
		for(i in 0...keyArray.length) {
			keyArray[i] = UP;
		}
		
		while (keyDownStack.length > 0) {
			keyDownStack.pop();
		}
	}
	
	private function resume()
	{
		isPaused = false;
	}
	
}