package mje;

import js.html.CanvasRenderingContext2D;
import mje.MJEData.Vector2;

/**
 * ...
 * @author JanGamesCo
 */
class MJEObject extends MJEBasic
{
	/** Position of the basic */
	public var position:Vector2;
	/** Velocity of the basic */
	public var velocity:Vector2;
	/** Acceleration of the basic */
	public var acceleration:Vector2;
	
	/** Alpha */
	public var alpha:Float;
	/** Color */
	public var color:String;
	
	/** X position */
	public var x(get, set):Float;
	/** Y position */
	public var y(get, set):Float;
	
	/** Width */
	public var width:Float;
	/** Height */
	public var height:Float;
	
	public function new(x:Float, y:Float)
	{
		super();
		
		//Position
		position = new Vector2(0, 0);
		this.x = x;
		this.y = y;
		
		//Velocity and acceleration
		velocity = new Vector2(0, 0);
		acceleration = new Vector2(0, 0);
		
		//Visibility and exists
		visible = false;
		exists = true;
		
		//Get ID
		ID = MJE.the.game.getID();
	}
	
	public function makeRect(width:Float, height:Float, color:String)
	{
		//Make visible
		visible = true;
		
		//Setup width and height
		this.width = width;
		this.height = height;
		
		//Setup color
		this.color = color;
	}
	
	override public function draw()
	{
		//Get context
		var ctx:CanvasRenderingContext2D = MJE.the.ctx;
		
		//Draw rect
		ctx.fillStyle = color;
		ctx.fillRect(x, y, width, height);
	}
	
	override public function update()
	{
		//Add acceleration to velocity
		velocity += acceleration * MJE.delta;
		
		//Add velocity to position
		position += velocity * MJE.delta;
	}
	
	/** AABB Collision detection */
	public function aabb(Other:MJEObject):Bool
	{		
		if (Other.exists && exists &&
			x < Other.x + Other.width &&
			x + width > Other.x &&
			y < Other.y + Other.height &&
			y + height > Other.y) {
			return true;
		}
		
		return false;
	}
	
	//X and Y getter and setters
	inline public function get_x():Float { return position.x; }
	inline public function set_x(v:Float):Float { return position.x = v; }
	inline public function get_y():Float { return position.y; }
	inline public function set_y(v:Float):Float { return position.y = v; }
	
}